@@ -1,8 +1,8 @@
-## PGLS.R (2020-04-13)
+## PGLS.R (2022-06-22)
 
 ##   Phylogenetic Generalized Least Squares
 
-## Copyright 2004-2020 Julien Dutheil, and 2006-2017 Emmanuel Paradis
+## Copyright 2004-2021 Julien Dutheil, and 2006-2017 Emmanuel Paradis
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -56,11 +56,11 @@ Initialize.corPhyl <- function(object, data, ...)
       warning("No covariate specified, species will be taken as ordered in the data frame. To avoid this message, specify a covariate containing the species names with the 'form' argument.")
     }
     ## Obtaining the group information, if any
-    if (!is.null(getGroupsFormula(form))) {
+    if (is.null(getGroupsFormula(form))) {
+        attr(object, "Dim") <- Dim(object, as.factor(rep(1, nrow(data))))
+    } else { # no groups
         attr(object, "groups") <- getGroups(object, form, data = data)
         attr(object, "Dim") <- Dim(object, attr(object, "groups"))
-    } else { # no groups
-        attr(object, "Dim") <- Dim(object, as.factor(rep(1, nrow(data))))
     }
     ## Obtaining the covariate(s)
     attr(object, "covariate") <- getCovariate(object, data = data)
@@ -78,6 +78,8 @@ corMatrix.corBrownian <-
     } else {
         tree <- attr(object, "tree")
         mat <- vcv.phylo(tree, corr = corr)
+        if (formula(object) == ~1) return(mat) # added by EP (2022-06-22)
+        covariate <- as.character(covariate)
         mat[covariate, covariate]
     }
 }
@@ -94,6 +96,8 @@ corMatrix.corMartins <-
         dist <- cophenetic.phylo(tree)
         mat <- exp(-object[1] * dist)
         if (corr) mat <- cov2cor(mat)
+        if (formula(object) == ~1) return(mat) # added by EP (2022-06-22)
+        covariate <- as.character(covariate)
         mat[covariate, covariate]
     }
 }
@@ -109,6 +113,8 @@ corMatrix.corGrafen <-
         tree <- compute.brlen(attr(object, "tree"),
                               method = "Grafen", power = exp(object[1]))
         mat <- vcv.phylo(tree, corr = corr)
+        if (formula(object) == ~1) return(mat) # added by EP (2022-06-22)
+        covariate <- as.character(covariate)
         mat[covariate, covariate]
     }
 }
@@ -205,11 +211,12 @@ corMatrix.corPagel <-
         as.list(lapply(covariate, function(el) corMatrix(object, covariate = el)))
     } else {
         mat <- vcv.phylo(attr(object, "tree"), corr = corr)
-        mat <- mat[covariate, covariate]
         tmp <- diag(mat)
         mat <- object[1]*mat
         diag(mat) <- tmp
-        mat
+        if (formula(object) == ~1) return(mat) # added by EP (2022-06-22)
+        covariate <- as.character(covariate)
+        mat[covariate, covariate]
     }
 }
 
@@ -250,6 +257,8 @@ probably need to set 'fixed = TRUE' in corBlomberg().")
         d <- (dist.nodes(phy)[length(phy$tip.label) + 1, ])^(1/object[1])
         phy$edge.length <- d[phy$edge[, 2]] - d[phy$edge[, 1]]
         mat <- vcv.phylo(phy, corr = corr)
+        if (formula(object) == ~1) return(mat) # added by EP (2022-06-22)
+        covariate <- as.character(covariate)
         mat[covariate, covariate]
     }
 }
