@@ -1,4 +1,4 @@
-## birthdeath.R (2021-10-23)
+## birthdeath.R (2022-02-19)
 
 ##   Estimation of Speciation and Extinction Rates
 ##             with Birth-Death Models
@@ -6,7 +6,7 @@
 ## birthdeath: standard model
 ## bd.ext: extended version
 
-## Copyright 2002-2021 Emmanuel Paradis
+## Copyright 2002-2022 Emmanuel Paradis
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -25,21 +25,21 @@ birthdeath <- function(phy)
               - 2 * sum(log(exp(r * x[2:N]) - a)))
     }
     out <- nlm(function(p) dev(p[1], p[2]), c(0.1, 0.2), hessian = TRUE)
+
+    getSEs <- function(x) {
+        H <- x$hessian
+        inv.hessian <- try(solve(H), silent = TRUE)
+        if (inherits(inv.hessian, "try-error")) return(rep(NA_real_, nrow(H)))
+        sqrt(diag(inv.hessian))
+    }
+
     if (out$estimate[1] < 0) {
         out <- nlm(function(p) dev(0, p), 0.2, hessian = TRUE)
         para <- c(0, out$estimate)
-        inv.hessian <- try(solve(out$hessian), silent = TRUE)
-        se <-
-            if (class(inv.hessian) == "try-error") NA
-            else sqrt(diag(inv.hessian))
-        se <- c(0, se)
-    }
-    else {
+        se <- c(0, getSEs(out))
+    } else {
         para <- out$estimate
-        inv.hessian <- try(solve(out$hessian), silent = TRUE)
-        se <-
-            if (class(inv.hessian) == "try-error") c(NA, NA)
-            else sqrt(diag(inv.hessian))
+        se <- getSEs(out)
     }
     Dev <- out$minimum
 
