@@ -19,6 +19,9 @@ centre <- function(D,n)
 {	One <- matrix(1,n,n)
 	mat <- diag(n) - One/n
 	mat.cen <- mat %*% D %*% mat
+        lowtri <- lower.tri(mat.cen)
+	mat.cen[lowtri] <- t(mat.cen)[lowtri]
+	mat.cen
 }
 
 bstick.def <- function (n, tot.var = 1, ...)   # 'bstick.default' from vegan
@@ -64,7 +67,7 @@ bstick.def <- function (n, tot.var = 1, ...)   # 'bstick.default' from vegan
 		k <- length(which(eig > epsilon))
 		rel.eig <- eig[1:k]/trace
 		cum.eig <- cumsum(rel.eig)
-		vectors <- sweep(D.eig$vectors[,1:k], 2, sqrt(eig[1:k]), FUN="*")
+		vectors <- sweep(D.eig$vectors[,1:k, drop = FALSE], 2, sqrt(eig[1:k]), FUN="*")
 		bs <- bstick.def(k)
 		cum.bs <- cumsum(bs)
 
@@ -90,7 +93,7 @@ bstick.def <- function (n, tot.var = 1, ...)   # 'bstick.default' from vegan
 		cum.eig.cor <- cumsum(rel.eig.cor)
 		k2 <- length(which(eig > epsilon))
 		k3 <- length(which(rel.eig.cor > epsilon))
-		vectors <- sweep(D.eig$vectors[,1:k2], 2, sqrt(eig[1:k2]), FUN="*")
+		vectors <- sweep(D.eig$vectors[, 1:k2, drop = FALSE], 2, sqrt(eig[1:k2]), FUN="*")
 		# Only the eigenvectors with positive eigenvalues are shown
 
 # Negative eigenvalues: three ways of handling the situation
@@ -129,7 +132,7 @@ bstick.def <- function (n, tot.var = 1, ...)   # 'bstick.default' from vegan
 		rel.eig.cor <- eig.cor[1:k]/trace.cor
 		cum.eig.cor <- cumsum(rel.eig.cor)
 		k2 <- length(which(eig.cor > epsilon))
-		vectors.cor <- sweep(toto.cor$vectors[,1:k2], 2, sqrt(eig.cor[1:k2]), FUN="*")
+		vectors.cor <- sweep(toto.cor$vectors[, 1:k2, drop = FALSE], 2, sqrt(eig.cor[1:k2]), FUN="*")
 		rownames(vectors.cor) <- names
 		colnames(vectors.cor) <- colnames(vectors.cor, do.NULL = FALSE, prefix = "Axis.")
 		# bs <- broken.stick(k2)[,2]
