@@ -1,8 +1,8 @@
-## write.tree.R (2023-02-03)
+## write.tree.R (2024-11-11)
 
 ##   Write Tree File in Parenthetic Format
 
-## Copyright 2002-2023 Emmanuel Paradis, Daniel Lawson, and Klaus Schliep
+## Copyright 2002-2024 Emmanuel Paradis, Daniel Lawson, and Klaus Schliep
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -66,10 +66,16 @@ write.tree <-
 
 .write.tree2 <- function(phy, digits = 10, tree.prefix = "", check_tips)
 {
-    brl <- !is.null(phy$edge.length)
+    brl <- (!is.null(phy$edge.length) && digits > 0)
     nodelab <- !is.null(phy$node.label)
     if (check_tips) phy$tip.label <- checkLabel(phy$tip.label)
-    if (nodelab) phy$node.label <- checkLabel(phy$node.label)
+    if (nodelab) {
+        ## fix by Martin Smith (2024-04-07)
+        if (length(phy[["node.label"]]) != phy[["Nnode"]]) {
+            warning("Length of node.label does not match number of nodes.")
+        }
+        phy$node.label <- checkLabel(phy$node.label)
+    }
     f.d <- paste0(":%.", digits, "g")
     n <- length(phy$tip.label)
 
@@ -87,16 +93,6 @@ write.tree <-
         if (!is.null(phy$root.edge)) tmp[1L] <- sprintf(f.d, phy$root.edge)
         INTS <- paste0(INTS, tmp)
     }
-###    INTS[1] <- paste0(INTS[1], ";")
-
-###    ## borrowed from phangorn:
-###    parent <- phy$edge[, 1]
-###    children <- phy$edge[, 2]
-###    kids <- vector("list", n + phy$Nnode)
-###    for (i in 1:length(parent))
-###        kids[[parent[i]]] <- c(kids[[parent[i]]], children[i])
-###    Nkids <- lengths(kids, FALSE)
-###    root <- parent[! parent %in% children][1]
 
     ## find the root node:
     tmp.nodes <- unique.default(phy$edge[, 1L])
@@ -106,13 +102,11 @@ write.tree <-
         stop("seems there is more than one root node")
     storage.mode(root) <- "integer"
 
-###    o <- postorder(phy)
     o <- reorderRcpp(phy$edge, n, root, 2L)
     ANC <- phy$edge[o, 1L]
     DESC <- phy$edge[o, 2L]
     NEWICK <- character(n + phy$Nnode)
     NEWICK[1:n] <- TERMS
-###    root <- n + 1L
     from <- to <- 1L
     repeat {
         thenode <- ANC[from]
@@ -127,5 +121,5 @@ write.tree <-
         if (thenode == root) break
         from <- to + 1L
     }
-    paste0(NEWICK[root], ";")
+    paste0(tree.prefix, NEWICK[root], ";")
 }
